# -*- coding: utf-8 -*-
"""Main Controller"""

import datetime

from tg import config, expose, flash, require, url, request, redirect
import transaction
from pylons.i18n import ugettext as _, lazy_ugettext as l_, ungettext as N_, \
        lazy_ungettext as lN_
from repoze.what import predicates
try:
    from sqlalchemy.orm import joinedload
except:
    from sqlalchemy.orm import eagerload as joinedload

from raffle.lib.base import BaseController
from raffle.lib.helpers import has_cla_plus_one
from raffle.model import DBSession
from raffle.controllers.admin import AdminController
from raffle.controllers.prize import PrizeController
from raffle.controllers.error import ErrorController
from raffle.model.main import Raffle
from raffle.model.ticket import Ticket
from raffle.model.prize import Prize

__all__ = ['RootController']


class RootController(BaseController):
    """
    The root controller for the raffle application.

    All the other controllers and WSGI applications should be mounted on this
    controller. For example::

        panel = ControlPanelController()
        another_app = AnotherWSGIApplication()

    Keep in mind that WSGI applications shouldn't be mounted directly: They
    must be wrapped around with :class:`tg.controllers.WSGIAppController`.

    """

    error = ErrorController()
    admin = AdminController()
    prizes = PrizeController()

    @expose('raffle.templates.index')
    def index(self):
        """Handle the front-page."""
        # Get list of raffles that ended a week ago or are ongoing
        end_time = datetime.datetime.utcnow() - datetime.timedelta(days=7)
        raffles = DBSession.query(Raffle).filter(Raffle.end_date >= end_time)
        return dict(raffles=raffles)

    @expose('raffle.templates.raffle')
    def raffle(self, raffle_id):
        raffle = DBSession.query(Raffle).filter(Raffle.id == raffle_id).one()
        return dict(raffle=raffle)

    @expose('raffle.templates.raffle')
    @require(predicates.not_anonymous())
    def enter_raffle(self, raffle_id):

        raffle = DBSession.query(Raffle).filter(Raffle.id == raffle_id).one()

        #
        # Error checking on the raffle
        #

        user_groups = frozenset(request.identity['groups'])

        # Check whether the raffle has expired
        now = datetime.datetime.utcnow()
        if raffle.end_date < now:
            flash(_('This raffle is over.  You can no longer enter'),
                    status='error')

        # Check whether the raffle hasn't begun yet
        elif raffle.start_date > now:
            flash(_('This raffle has not yet begun.  Please wait before'
                ' entering.'), status='error')

        # Check that the user hasn't submitted more than the limit of
        # tickets yet
        elif (raffle.ticket_limit and
                request.identity['user'].user_name in raffle.entrants and
                raffle.ticket_limit <=
                len(raffle.entrants[request.identity['user'].user_name])):
            flash(N_('You may only submit %(limit)s ticket in this raffle',
                'You may only submit %(limit)s tickets in this raffle',
                raffle.ticket_limit) % {'limit': raffle.ticket_limit},
                status='error')

        # Check that the user is in a group that is eligible
        elif raffle.groups and (
                not (user_groups.intersection(raffle.groups) or
                ('cla + one' in raffle.groups and has_cla_plus_one()))):
            flash(N_('To be eligible for this raffle, you must be in group:'
                    ' %(groups)s',
                    'To be eligible for this raffle, you must be in one of'
                    ' the following groups: %(groups)s', len(raffle.groups))
                    % {'groups': ', '.join(raffle.groups)}, status='error')

        # Eligible
        else:
            tkt = Ticket(person=request.identity['user'].user_name,
                    raffle_id=raffle_id)
            DBSession.add(tkt)
            # ORM mapped raffle object will be invalid following the commit()
            raffle_title = raffle.title
            try:
                transaction.commit()
            except:
                flash(_('Saving your entry did not succeed.  Please try again.'
                    '  If this persists, please contact the administrators at'
                    ' %(email)s') % {'email': config.get('serveradmin.email')},
                    status='error')
            else:
                flash(_('You have successfully entered raffle: %(title)s' %
                    {'title': raffle_title}))

        redirect('/raffle/%s' % raffle_id)

    @expose('raffle.templates.results')
    def results(self, raffle_id):
        raffle = DBSession.query(Raffle).filter(Raffle.id==raffle_id).one()
        tkts = DBSession.query(Ticket).filter(Ticket.raffle_id==raffle_id)
        num_tkts = tkts.count()
        tkt_owners = set()
        for ticket in tkts:
            tkt_owners.add(ticket.person)
        num_tkt_owners = len(tkt_owners)
        prizes = DBSession.query(Prize).options(joinedload('winner'))
        return dict(raffle=raffle, num_tkts=num_tkts,
                num_tkt_owners=num_tkt_owners, prizes=prizes)


    @expose('raffle.templates.mytickets')
    @require(predicates.not_anonymous())
    def mytickets(self):
        tickets = DBSession.query(Ticket)\
                .filter(Ticket.person==request.identity['user'].user_name)
        return dict(tickets=tickets)

    @expose('raffle.templates.login')
    def login(self, came_from='/'):
        """Start the user login."""
        login_counter = request.environ['repoze.who.logins']
        if login_counter > 0:
            flash(_('Wrong credentials'), 'warning')
        return dict(page='login', login_counter=str(login_counter),
                    came_from=came_from)

    @expose()
    def post_login(self, came_from='/'):
        """
        Redirect the user to the initially requested page on successful
        authentication or redirect her back to the login page if login failed.

        """
        if not request.identity:
            login_counter = request.environ['repoze.who.logins'] + 1
            redirect('/login', came_from=came_from, __logins=login_counter)
        userid = request.identity['repoze.who.userid']
        flash(_('Welcome back, %s!') % userid)
        redirect(came_from)

    @expose()
    def post_logout(self, came_from='/'):
        """
        Redirect the user to the initially requested page on logout and say
        goodbye as well.

        """
        flash(_('We hope to see you soon!'))
        redirect(came_from)
