# -*- coding: utf-8 -*-
'''Sample model module.'''
import operator

from sqlalchemy import ForeignKey, Column
from sqlalchemy.orm import backref
try:
    from sqlalchemy.orm import relationship
except ImportError:
    from sqlalchemy.orm import relation as relationship
from sqlalchemy.orm.collections import MappedCollection, \
        collection, attribute_mapped_collection
from sqlalchemy.types import Integer, UnicodeText, Boolean, DateTime

from raffle.model import DeclarativeBase
from raffle.model import ticket, prize

class RaffleEntrantCollectionMap(MappedCollection):
    '''
    Map raffles to entrants.  Needed because each entrant should point to a
    list of their tickets
    '''
    @collection.internally_instrumented
    def __setitem__(self, key, value, _sa_initiator=None):
        if key in self:
            new_value = self[key]
            new_value.append(value)
        else:
            new_value = [value]
        super(RaffleEntrantCollectionMap, self).__setitem__(key, new_value,
                _sa_initiator)

def attribute_RaffleEntrantCollectionMap(attr_name):
    return lambda: RaffleEntrantCollectionMap(operator.attrgetter(attr_name))

class RaffleGroupSet(set):
    __emulates__ = set

    def add(self, item):
        super(RaffleGroupSet, self).add(item.group)

class Raffle(DeclarativeBase):
    __tablename__ = 'raffle'

    id = Column(Integer, primary_key=True)

    title = Column(UnicodeText, nullable=False, unique=True)
    description = Column(UnicodeText, nullable=False)
    restrictions = Column(UnicodeText, nullable=True)
    ticket_limit = Column(Integer, nullable=True, default=1)
    winning_limit = Column(Integer, nullable=True, default=1)
    start_date = Column(DateTime, nullable=False)
    end_date = Column(DateTime, nullable=False)
    auto_winner = Column(Boolean, nullable=False, default=True)

    tickets = relationship (ticket.Ticket, backref=backref('raffle'),
            collection_class=attribute_mapped_collection('id'))
    # Check what happens with multiple tickets.
    # We want it to generate a list of tickets.  May need a custom
    # collection_class for that
    entrants = relationship(ticket.Ticket,
            collection_class=attribute_RaffleEntrantCollectionMap('person'))
    extra_entrant_data = relationship(ticket.PeopleData,
            backref=backref('raffle'),
            collection_class=attribute_mapped_collection('person'))
    prizes = relationship(prize.Prize,
            backref=backref('raffle'))

# Special groups these contain spaces:
# 'cla + one' -- cla + one other group

class RaffleGroup(DeclarativeBase):
    __tablename__ = 'raffle_group'

    raffle_id = Column(Integer, ForeignKey('raffle.id'))
    group = Column(UnicodeText, nullable=False)

    raffle = relationship(Raffle, backref=backref('groups', order_by=group,
        collection_class=RaffleGroupSet))

    __mapper_args__ = {'primary_key': (raffle_id, group)}
